@@ -1,6 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
+    Copyright (C) 2007-2009 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,7 +37,7 @@
 using namespace std;
 
 
-struct _GnomeCmdBookmarkDialogPrivate
+struct GnomeCmdBookmarkDialogPrivate
 {
     GList *groups;
     GnomeCmdBookmark *sel_bookmark;
@@ -61,23 +62,110 @@ guint bookmark_dialog_default_column_width[BOOKMARK_DIALOG_NUM_COLUMNS] = {
 };
 
 
-static void
-show_bookmark_dialog (const gchar *name, const gchar *path,
-                      const gchar *title,
-                      GnomeCmdStringDialogCallback on_ok,
-                      GnomeCmdBookmarkDialog *dialog)
+static void response_callback (GtkDialog *dialog, int response_id, gpointer unused)
 {
-    const gchar *labels[] = {
-        _("Bookmark name:"),
-        _("Bookmark target:"),
-    };
+    switch (response_id)
+    {
+        case GTK_RESPONSE_OK:
+            {
+                const gchar *name = gtk_entry_get_text (GTK_ENTRY (lookup_widget (GTK_WIDGET (dialog), "name")));
 
-    GtkWidget *dlg = gnome_cmd_string_dialog_new (title, labels, 2, (GnomeCmdStringDialogCallback)on_ok, dialog);
+                if (!name || !*name)
+                {
+                    g_signal_stop_emission_by_name (dialog, "response");
+                    gnome_cmd_show_message (GTK_WINDOW (dialog), _("Bookmark name is missing"));
+                    break;
+                }
 
-    gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dlg), 0, name);
-    gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dlg), 1, path);
+                const gchar *path = gtk_entry_get_text (GTK_ENTRY (lookup_widget (GTK_WIDGET (dialog), "path")));
 
-    gtk_widget_show (dlg);
+                if (!path || !*path)
+                {
+                    g_signal_stop_emission_by_name (dialog, "response");
+                    gnome_cmd_show_message (GTK_WINDOW (dialog), _("Bookmark target is missing"));
+                    break;
+                }
+            }
+
+            break;
+
+        case GTK_RESPONSE_NONE:
+        case GTK_RESPONSE_DELETE_EVENT:
+        case GTK_RESPONSE_CANCEL:
+            break;
+
+        default :
+            g_assert_not_reached ();
+    }
+}
+
+
+static gboolean edit_bookmark_dialog (GtkWindow *parent, const gchar *title, gchar *&name, gchar *&path)
+{
+    GtkWidget *dialog = gtk_dialog_new_with_buttons (title, parent,
+                                                     GtkDialogFlags (GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
+                                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                                     GTK_STOCK_OK, GTK_RESPONSE_OK,
+                                                     NULL);
+
+    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+
+    // HIG defaults
+    gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+    gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), 5);
+    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->action_area),6);
+
+    GtkWidget *table, *label, *entry;
+
+    table = gtk_table_new (3, 2, FALSE);
+    gtk_container_set_border_width (GTK_CONTAINER (table), 5);
+    gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+    gtk_table_set_col_spacings (GTK_TABLE (table), 12);
+    gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), table);
+
+    label = gtk_label_new_with_mnemonic (_("Bookmark _name:"));
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
+
+    entry = gtk_entry_new ();
+    gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
+    gtk_entry_set_text (GTK_ENTRY (entry), name);
+    g_object_set_data (G_OBJECT (dialog), "name", entry);
+    gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+    gtk_table_attach_defaults (GTK_TABLE (table), entry, 1, 2, 0, 1);
+
+    label = gtk_label_new_with_mnemonic (_("Bookmark _target:"));
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
+
+    entry = gtk_entry_new ();
+    gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
+    gtk_entry_set_text (GTK_ENTRY (entry), path);
+    g_object_set_data (G_OBJECT (dialog), "path", entry);
+    gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+    gtk_table_attach_defaults (GTK_TABLE (table), entry, 1, 2, 1, 2);
+
+    gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
+
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+    g_signal_connect (dialog, "response", G_CALLBACK (response_callback), NULL);
+
+    gint result = gtk_dialog_run (GTK_DIALOG (dialog));
+
+    if (result==GTK_RESPONSE_OK)
+    {
+        g_free (name);
+        name = g_strdup (gtk_entry_get_text (GTK_ENTRY (lookup_widget (GTK_WIDGET (dialog), "name"))));
+        g_free (path);
+        path = g_strdup (gtk_entry_get_text (GTK_ENTRY (lookup_widget (GTK_WIDGET (dialog), "path"))));
+    }
+
+    gtk_widget_destroy (dialog);
+
+    return result==GTK_RESPONSE_OK;
 }
 
 
@@ -143,45 +231,17 @@ static void on_dir_remove (GtkButton *button, GnomeCmdBookmarkDialog *dialog)
 }
 
 
-static gboolean on_edit_ok (GnomeCmdStringDialog *string_dialog, const gchar **values, GnomeCmdBookmarkDialog *dialog)
-{
-    const gchar *name = values[0];
-    const gchar *path = values[1];
-    GtkCList *dir_list = GTK_CLIST (dialog->priv->dir_list);
-
-    g_return_val_if_fail (dialog->priv->sel_bookmark != NULL, TRUE);
-
-    if (!name)
-    {
-        gnome_cmd_string_dialog_set_error_desc (string_dialog, g_strdup (_("Bookmark name is missing")));
-        return FALSE;
-    }
-
-    if (!path)
-    {
-        gnome_cmd_string_dialog_set_error_desc (string_dialog, g_strdup (_("Bookmark target is missing")));
-        return FALSE;
-    }
-
-    g_free (dialog->priv->sel_bookmark->name);
-    dialog->priv->sel_bookmark->name = g_strdup (name);
-    g_free (dialog->priv->sel_bookmark->path);
-    dialog->priv->sel_bookmark->path = g_strdup (path);
-
-    gtk_clist_set_text (dir_list, dir_list->focus_row, 1, name);
-    gtk_clist_set_text (dir_list, dir_list->focus_row, 2, path);
-
-    return TRUE;
-}
-
-
 static void on_dir_edit (GtkButton *button, GnomeCmdBookmarkDialog *dialog)
 {
-    show_bookmark_dialog (dialog->priv->sel_bookmark->name,
-                          dialog->priv->sel_bookmark->path,
-                          _("Edit Bookmark"),
-                          (GnomeCmdStringDialogCallback)on_edit_ok,
-                          dialog);
+    g_return_if_fail (dialog->priv->sel_bookmark != NULL);
+
+    if (edit_bookmark_dialog (GTK_WINDOW (dialog), _("Edit Bookmark"), dialog->priv->sel_bookmark->name, dialog->priv->sel_bookmark->path))
+    {
+        GtkCList *dir_list = GTK_CLIST (dialog->priv->dir_list);
+
+        gtk_clist_set_text (dir_list, dir_list->focus_row, 1, dialog->priv->sel_bookmark->name);
+        gtk_clist_set_text (dir_list, dir_list->focus_row, 2, dialog->priv->sel_bookmark->path);
+    }
 }
 
 
@@ -212,7 +272,8 @@ static void on_dir_move_up (GtkButton *button, GnomeCmdBookmarkDialog *dialog)
 {
     GtkCList *dir_list = GTK_CLIST (dialog->priv->dir_list);
 
-    if (dir_list->focus_row >= 1) {
+    if (dir_list->focus_row >= 1)
+    {
         gtk_clist_row_move (dir_list, dir_list->focus_row, dir_list->focus_row-1);
         update_move_buttons (dialog, dir_list->focus_row);
     }
@@ -223,7 +284,8 @@ static void on_dir_move_down (GtkButton *button, GnomeCmdBookmarkDialog *dialog)
 {
     GtkCList *dir_list = GTK_CLIST (dialog->priv->dir_list);
 
-    if (dir_list->focus_row >= 0) {
+    if (dir_list->focus_row >= 0)
+    {
         gtk_clist_row_move (dir_list, dir_list->focus_row, dir_list->focus_row+1);
         update_move_buttons (dialog, dir_list->focus_row);
     }
@@ -326,7 +388,7 @@ static void add_groups (GnomeCmdBookmarkDialog *dialog)
 
 static void add_bookmarks (GnomeCmdBookmarkDialog *dialog)
 {
-    GnomeCmdCon *current_con = gnome_cmd_file_selector_get_connection (gnome_cmd_main_win_get_fs (main_win, ACTIVE));
+    GnomeCmdCon *current_con = gnome_cmd_main_win_get_fs (main_win, ACTIVE)->get_connection();
 
     g_return_if_fail (current_con != NULL);
 
@@ -479,26 +541,26 @@ static void init (GnomeCmdBookmarkDialog *in_dialog)
     gtk_widget_set_sensitive (GTK_WIDGET (in_dialog->priv->remove_btn), FALSE);
 
     in_dialog->priv->move_up_btn = create_named_stock_button (
-        dialog, GNOME_STOCK_BUTTON_UP, "move_up_button",
+        dialog, GTK_STOCK_GO_UP, "move_up_button",
         GTK_SIGNAL_FUNC (on_dir_move_up));
     GTK_WIDGET_SET_FLAGS (in_dialog->priv->move_up_btn, GTK_CAN_DEFAULT);
     gtk_box_pack_start (GTK_BOX (bbox), in_dialog->priv->move_up_btn, FALSE, TRUE, 0);
     gtk_widget_set_sensitive (GTK_WIDGET (in_dialog->priv->move_up_btn), FALSE);
 
     in_dialog->priv->move_down_btn = create_named_stock_button (
-        dialog, GNOME_STOCK_BUTTON_DOWN, "move_down_button",
+        dialog, GTK_STOCK_GO_DOWN, "move_down_button",
         GTK_SIGNAL_FUNC (on_dir_move_down));
     GTK_WIDGET_SET_FLAGS (in_dialog->priv->move_down_btn, GTK_CAN_DEFAULT);
     gtk_box_pack_start (GTK_BOX (bbox), in_dialog->priv->move_down_btn, FALSE, TRUE, 0);
     gtk_widget_set_sensitive (GTK_WIDGET (in_dialog->priv->move_down_btn), FALSE);
 
 
-    gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), GNOME_STOCK_BUTTON_CLOSE,
+    gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), GTK_STOCK_CLOSE,
                                  GTK_SIGNAL_FUNC (on_close), dialog);
 
     gtk_signal_connect (GTK_OBJECT (dialog), "key-press-event",
                         GTK_SIGNAL_FUNC (on_dialog_keypress), dialog);
-    gtk_signal_connect_after (GTK_OBJECT (in_dialog->priv->dir_list), "scroll_vertical",
+    gtk_signal_connect_after (GTK_OBJECT (in_dialog->priv->dir_list), "scroll-vertical",
                               GTK_SIGNAL_FUNC (on_scroll_vertical), NULL);
     gtk_signal_connect (GTK_OBJECT (in_dialog->priv->dir_list), "unselect-row",
                         GTK_SIGNAL_FUNC (on_dir_unselected), dialog);
@@ -515,7 +577,7 @@ static void init (GnomeCmdBookmarkDialog *in_dialog)
  * Public functions
  ***********************************/
 
-GtkType gnome_cmd_bookmark_dialog_get_type (void)
+GtkType gnome_cmd_bookmark_dialog_get_type ()
 {
     static GtkType dlg_type = 0;
 
@@ -539,7 +601,7 @@ GtkType gnome_cmd_bookmark_dialog_get_type (void)
 }
 
 
-GtkWidget *gnome_cmd_bookmark_dialog_new (void)
+GtkWidget *gnome_cmd_bookmark_dialog_new ()
 {
     GnomeCmdBookmarkDialog *dialog = (GnomeCmdBookmarkDialog *) gtk_type_new (gnome_cmd_bookmark_dialog_get_type ());
 
@@ -552,42 +614,41 @@ GtkWidget *gnome_cmd_bookmark_dialog_new (void)
 }
 
 
-static gboolean on_new_bookmark_ok (GnomeCmdStringDialog *string_dialog, const gchar **values, gpointer data)
+void gnome_cmd_bookmark_add_current ()
 {
-    GnomeCmdBookmark *bookmark = g_new0 (GnomeCmdBookmark, 1);
-    GnomeCmdCon *con = gnome_cmd_file_selector_get_connection (gnome_cmd_main_win_get_fs (main_win, ACTIVE));
-    GnomeCmdBookmarkGroup *group = gnome_cmd_con_get_bookmarks (con);
-
-    bookmark->name = g_strdup (values[0]);
-    bookmark->path = g_strdup (values[1]);
-    bookmark->group = group;
-
-    group->bookmarks = g_list_append (group->bookmarks, bookmark);
-
-    gnome_cmd_main_win_update_bookmarks (main_win);
-
-    return TRUE;
-}
-
-
-void gnome_cmd_bookmark_add_current (void)
-{
-    GnomeCmdDir *cwd = gnome_cmd_file_selector_get_directory (gnome_cmd_main_win_get_fs (main_win, ACTIVE));
-    gchar *path = gnome_cmd_file_get_path (GNOME_CMD_FILE (cwd));
+    GnomeCmdFileSelector *fs = gnome_cmd_main_win_get_fs (main_win, ACTIVE);
+    GnomeCmdDir *cwd = fs->get_directory();
+    gchar *path = gnome_cmd_dir_is_local (cwd) ? gnome_cmd_file_get_real_path (GNOME_CMD_FILE (cwd)) :
+                                                 gnome_cmd_file_get_path (GNOME_CMD_FILE (cwd));
 
     if (!g_utf8_validate (path, -1, NULL))
     {
-        create_error_dialog (_("To bookmark a directory the whole search path to the directory must be in valid UTF-8 encoding\n"));
+        gnome_cmd_show_message (NULL, _("To bookmark a directory the whole search path to the directory must be in valid UTF-8 encoding"));
         g_free (path);
         return;
     }
 
-    show_bookmark_dialog (g_basename (path),
-                          path,
-                          _("New Bookmark"),
-                          (GnomeCmdStringDialogCallback)on_new_bookmark_ok,
-                          NULL);
-    g_free (path);
+    gchar *name = g_strdup (g_basename (path));
+
+    if (edit_bookmark_dialog (NULL, _("New Bookmark"), name, path))
+    {
+        GnomeCmdCon *con = fs->is_local() ? get_home_con () : fs->get_connection();
+        GnomeCmdBookmarkGroup *group = gnome_cmd_con_get_bookmarks (con);
+        GnomeCmdBookmark *bookmark = g_new0 (GnomeCmdBookmark, 1);
+
+        bookmark->name = name;
+        bookmark->path = path;
+        bookmark->group = group;
+
+        group->bookmarks = g_list_append (group->bookmarks, bookmark);
+
+        gnome_cmd_main_win_update_bookmarks (main_win);
+    }
+    else
+    {
+        g_free (name);
+        g_free (path);
+    }
 }
 
 
@@ -598,10 +659,10 @@ void gnome_cmd_bookmark_goto (GnomeCmdBookmark *bookmark)
     GnomeCmdFileSelector *fs = gnome_cmd_main_win_get_fs (main_win, ACTIVE);
     g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
 
-    GnomeCmdCon *current_con = gnome_cmd_file_selector_get_connection (fs);
+    GnomeCmdCon *current_con = fs->get_connection();
 
     if (current_con == bookmark->group->con)
-        gnome_cmd_file_selector_goto_directory (fs, bookmark->path);
+        fs->goto_directory(bookmark->path);
     else
     {
         GnomeCmdCon *con = bookmark->group->con;
@@ -609,17 +670,17 @@ void gnome_cmd_bookmark_goto (GnomeCmdBookmark *bookmark)
         if (con->state == CON_STATE_OPEN)
         {
             GnomeCmdDir *dir = gnome_cmd_dir_new (con, gnome_cmd_con_create_path (con, bookmark->path));
-            gnome_cmd_file_selector_set_connection (fs, con, dir);
+            fs->set_connection(con, dir);
 
         }
         else
         {
             if (con->base_path)
-                gtk_object_unref (GTK_OBJECT (con->base_path));
+                g_object_unref (con->base_path);
 
             con->base_path = gnome_cmd_con_create_path (con, bookmark->path);
             gtk_object_ref (GTK_OBJECT (con->base_path));
-            gnome_cmd_file_selector_set_connection (fs, con);
+            fs->set_connection(con);
         }
     }
 }
